# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Starts services for a Master to be controlled.
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import socket
from optparse import OptionParser
import logging

from master import Master
from masterctrlserver import MasterCtrlServer
from mapwoc.lib.configutil import get_config_file_entry

def setup_logging(quiet, verbose):
    """Setup logging."""

    mainlogger = logging.getLogger('masterctrl')
    mainlogger.addHandler(logging.FileHandler('master.log'))
    mainlogger.addHandler(logging.StreamHandler())
    masterlogger = logging.getLogger('master')
    masterlogger.addHandler(logging.FileHandler('master.log'))
    masterlogger.addHandler(logging.StreamHandler())
    serverlogger = logging.getLogger('masterctrlsrv')
    serverlogger.addHandler(logging.FileHandler('master.log'))
    serverlogger.addHandler(logging.StreamHandler())

    if verbose:
        mainlogger.setLevel(logging.DEBUG)
        masterlogger.setLevel(logging.DEBUG)
        serverlogger.setLevel(logging.DEBUG)
    elif quiet:
        mainlogger.setLevel(logging.WARNING)
        masterlogger.setLevel(logging.WARNING)
        serverlogger.setLevel(logging.WARNING)
    else:
        mainlogger.setLevel(logging.INFO)
        masterlogger.setLevel(logging.INFO)
        serverlogger.setLevel(logging.INFO)

def ask_redirector(question):
    host,port = 'localhost', 8228
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((host,port))
        sock.send(question)
        received = sock.recv(1024)
        sock.close()
    except socket.error, msg:
        #logger.warning("Error when talking to redirector: %s" % msg)
        return ''
    return received

def poll():
    while True:
        time.sleep(4)
    pass

if __name__ == "__main__":
    """Starts a MasterCtrlServer.
    Information about node connections is read from a directory.
    """

    parser = OptionParser()
    parser.add_option("", "--config", dest="configfile", help="Path to configuration file.",
                      default="/usr/share/wazmasterctrl/config")
    parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                      help="Be verbose and chatty on stdout and logfile.")
    parser.add_option("-q", "--quiet", dest="quiet", action="store_true",
                      help="Be quiet, only print warnings and"
                           " errors to stdout and logfile.")
    (options, args) = parser.parse_args()
    if options.quiet and options.verbose:
        print "--verbose takes precedence over --quiet"

    setup_logging(options.quiet, options.verbose)
    logger = logging.getLogger('masterctrl')

    master = Master(options.configfile)
    port = get_config_file_entry(options.configfile, 'config', 'ctrl_port')

    server = MasterCtrlServer((socket.gethostname(),int(port)),master)
    logger.info('Starting server.')
    #poller = multiprocessing.Process
    server.serve_forever()

