# -*- coding: utf-8 -*-

# mapWOC
#
# Description: NodeCtrl - start services for a node to be controlled.
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

from optparse import OptionParser
import socket
import logging

from ConfigParser import NoSectionError
from twisted.internet import reactor, task

from mapwoc.lib.configutil import get_config_file_entry
from nodectrlserver import NodeCtrlServer
from mapwoc.nodectrl.node import Node
from twisted_node import NodeResponseFactory

def setup_logging(quiet, verbose):
    """Setup logging."""

    mainlogger = logging.getLogger('nodectrl')
    mainlogger.addHandler(logging.FileHandler('node.log'))
    mainlogger.addHandler(logging.StreamHandler())
    nodelogger = logging.getLogger('node')
    nodelogger.addHandler(logging.FileHandler('node.log'))
    nodelogger.addHandler(logging.StreamHandler())
    serverlogger = logging.getLogger('nodectrlsrv')
    serverlogger.addHandler(logging.FileHandler('node.log'))
    serverlogger.addHandler(logging.StreamHandler())

    if verbose:
        mainlogger.setLevel(logging.DEBUG)
        nodelogger.setLevel(logging.DEBUG)
        serverlogger.setLevel(logging.DEBUG)
    elif quiet:
        mainlogger.setLevel(logging.WARNING)
        nodelogger.setLevel(logging.WARNING)
        serverlogger.setLevel(logging.WARNING)
    else:
        mainlogger.setLevel(logging.INFO)
        nodelogger.setLevel(logging.INFO)
        serverlogger.setLevel(logging.INFO)


# Main function.
if __name__ == "__main__":

    parser = OptionParser()
    parser.add_option("", "--config", dest="configfile",
                      help="Path to configuration file.",
                      default="/usr/share/waznodectrl/node.config")
    parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                      help="Be verbose and chatty on stdout and logfile.")
    parser.add_option("-q", "--quiet", dest="quiet", action="store_true",
                      help="Be quiet, only print warnings and"
                           " errors to stdout and logfile.")
    (options, args) = parser.parse_args()
    if options.quiet and options.verbose:
        print "--verbose takes precedence over --quiet"

    setup_logging(options.quiet, options.verbose)
    logger = logging.getLogger('nodectrl')

    try:
        node = Node(options.configfile)

        logger.info("Done. Loaded %d HCs." % len(node.hcs))
        port = get_config_file_entry(options.configfile, 'config',
                                     'ctrl_port')

        host = get_config_file_entry(options.configfile, 'config',
                                     'ctrl_interface')
        factory = NodeResponseFactory(node)
        reactor.listenTCP(port, factory)
        node_poll_hcs = task.LoopingCall(node.poll_hcs)
        node_poll_hcs.start(5)

        logger.info("NodeCtrl Server will attempt to listen on %s" % port)

        # TODO cleanup old copies of images, dump files etc (and emit warning).
        reactor.run()
    except NoSectionError:
        logger.error("Could not parse configuration file %s ." % options.configfile)
        exit(1)
    except OSError as ex:
        logger.error("OSError({0}): {1} - {2}".format(ex.errno, ex.strerror,
                                               ex.filename))
        exit(1)
    except KeyboardInterrupt:
        logger.warning("NodeCtrl was interrupted.")
        reactor.stop()
        exit(0)

